#!/usr/bin/perl
#
# pcost.cgi
#
# Druckkostenverwaltung
#
# Joachim Deckers <deckersj@mariengymnasium-arnsberg.de>, 07.10.2000
#

use strict;
use CGI qw/:standard/;


# ----------------------------------
package job;

local ($job::counter);

sub new {
	my $class=shift;
	my $self = {};
	$self->{'tag'}=shift;
	$self->{'zeit'}=shift;
	$self->{'vorher'}=shift;
	$self->{'nachher'}=shift;
	$self->{'naechster'}=shift;
	if ($self->{'vorher'}>=0) {
		if ($self->{'naechster'}>=0) {
			$self->{'gedruckt'}=$self->{'naechster'}-$self->{'vorher'};
		} else {
			if ($self->{'nachher'}>=0) {
				$self->{'gedruckt'}=$self->{'nachher'}-$self->{'vorher'};
				if ($self->{'gedruckt'}==0) {$self->{'gedruckt'}='unbekannte Anzahl';}
			} else {
				$self->{'gedruckt'}='unbekannte Anzahl';
			}
		}			
	} else {
		$self->{'gedruckt'}='unbekannte Anzahl';	
	}
		
	$job::counter++;
	bless($self,$class);
}

sub ausgabe {
	my $self=shift;
	my $ausfuehrlich = shift;
	$ausfuehrlich = 0 if not defined $ausfuehrlich;
	
	if (! $ausfuehrlich) { return "  $self->{'tag'}, $self->{'zeit'}: $self->{'gedruckt'} Seiten\n"; }
	else { return "  $self->{'tag'}, $self->{'zeit'}: $self->{'gedruckt'} Seiten ($self->{'vorher'},$self->{'nachher'}|$self->{'naechster'})\n"; }
}

sub gedruckt {
	my $self=shift;
	return ($self->{'gedruckt'} !~ 'unbek') ? $self->{'gedruckt'} : 0;
}

sub DESTROY { $job::counter-- };

# ----------------------------------
package userentry;

local ($userentry::counter);

sub new {
	my $class=shift;
	my $self = {};
	$self->{'name'}=shift;
	$self->{'bezahlt'}=shift;
	$self->{'gedruckt'}=0;
	$self->{'anzjobs'}=0;
	$self->{'jobs'}=();
	$userentry::counter++;
	bless($self,$class);
}

sub einzahlung {
	my $self = shift;
	my $anz = shift;
	$self->{'bezahlt'}+=$anz;
}

sub neuerjob {
	my $self = shift;
	my $job = shift;
	my $num = $self->{'anzjobs'};
	$self->{'jobs'}[$num]=$job;
	$self->{'gedruckt'} += ($self->{'jobs'}[$num])->gedruckt();
	$self->{'anzjobs'}++;
}

sub ausgabe {
	my $self=shift;
	my $AUS=shift;
	my $ausfuehrlich=shift;
	my ($i,$job,$ret);
	
	$ausfuehrlich=0 if not defined $ausfuehrlich;
	$ret = "User: ".$self->{'name'}."\n".
	"  Alter Kontostand : ".$self->{'bezahlt'}." Seiten.\n".
	"  Danach gedruckt  : ".$self->{'gedruckt'}." Seiten in ".$self->{'anzjobs'}." Druckauftrgen.\n".
	"  Neuer Kontostand : ".($self->{'bezahlt'}-$self->{'gedruckt'})." Seiten.\n".
	"  Liste der Druckauftrge:\n";
	for ($i=0;$i<$self->{'anzjobs'};++$i) {
		$job = $self->{'jobs'}[$i];
		$ret = $ret.($job->ausgabe($ausfuehrlich));
	}
	return $ret;
}

sub DESTROY { $userentry::counter-- };

# ----------------------------------
package main;

# Aufbau des Hashs:
# username, [username, bezahlt, seitengedruckt, anzeintrge, eintrag1, eintrag2, ...]
#  Eintrag:   [tag, zeit, seitenstart, seitenende, naechsterstart]

my %USER;
my $logfilename = "/var/log/pcost/pc-log";
my $dbfilename = "/var/local/pcost-db";
my ($user,$einzahlung,$startend,$zeitstempel,$pc,$ostartend,$ouser,$opc,$nachher,$firstpc,$lastpc,$j,$m,$t,$tag,$zeit,$stand,$summe);
my $sendmail = "/usr/sbin/sendmail";


# Hash mit Daten aus der Datenbank fllen
if (open DB,"<$dbfilename") {
	while (<DB>) {
		chomp;
		if (! /^#/) {
			($user,$einzahlung) = split /\t/;
			if (not defined $USER{"$user"}) {
				$USER{"$user"}=userentry->new($user,$einzahlung);
			} else {
				$USER{"$user"}->{'bezahlt'} += $einzahlung;
			}
		}
	}
	close DB;
} else {
	print STDERR "Konnte Datenbank nicht ffnen: ".$dbfilename."\n";
}

$ouser=""; $ostartend=""; $opc=0; $nachher=0;

# Hash mit Daten aus dem Logbuch fllen
if (open LOGF,"<$logfilename") {
	while (<LOGF>) {
		chomp;
		if (! /^#/) { # Kommentarzeilen berlesen!
			($startend,$user,$zeitstempel,$pc) = split /\t/;
			if (not defined $firstpc) { $firstpc=$pc; }
			$lastpc=$pc;
			$stand=$zeitstempel;
			($j,$m,$t,$zeit)=split /\-/,$zeitstempel,4;
			$tag="$t.$m.$j";
			$zeit=~s/\.\d\d\d$//;
			if (not defined $USER{"$user"}) { $USER{"$user"}=userentry->new($user,0); }
			if ($ostartend =~ /start/) { # Job ist gestartet worden 
			  if ($startend =~ /start/) { # erneuter Jobstart 
			    $USER{"$ouser"}->neuerjob(job->new("$tag","$zeit",$opc,$nachher,$pc));
			    $ouser=$user;$opc=$pc;
			  } else { # Jobende
			      $nachher=$pc;	
			      $ouser=$user;
			  }
			} else {
				$ouser=$user; $ostartend=$startend;$opc=$pc;
			}
		}
	}
	# Den letzten Job auch erfassen!!!	
	if ($ostartend =~ /start/) { # Job ist gestartet worden 
	  $USER{"$ouser"}->neuerjob(job->new("$tag","$zeit",$opc,$nachher,$pc));
	} 
	close LOGF;
} else {
	print STDERR "Konnte Logbuch nicht ffnen: ".$logfilename."\n";
}

# Und nun zur eigentlichen Druckkostenverwaltung


my $q = new CGI;

# Benutzerkennung merken
my $REMOTE = $q->remote_user();
my $HOST=$q->remote_host();

# HTML Kopf ausgeben
print $q->header,
      $q->start_html(-title=>'Druckkostenverwaltung - Mariengymnasium Arnsberg',
        -author=>'Joachim Deckers <deckersj@mariengymnasium-arnsberg>',
        -BACKGROUND=>'http://arktur/whttxtr2.jpg'
        );

if ( "$REMOTE" eq "" ) { 
	print $q->p('<b>Die Druckkostenverwaltung steht nur nach erfolgreicher Authentifizierung mit Benutzername und Kennwort zur Verf&uuml;gung!</b></BODY></HTML>');
	exit 0;
};

my ($name,$passwd,$uid,$gid,$quota,$comment,$gcos,$dir,$shell,$expire) = getpwnam ($REMOTE);
print $q->h2("Druckkostenverwaltung f&uuml;r den Laserdrucker im Computerraum"),
      "Sie sind angemeldet als <font color=darkred><b>".(split (/,/,$gcos))[0]." ($REMOTE)</b></font>";

menue() if (not defined $q->param('ausgewaehlt'));
auswerten() if ($q->param('ausgewaehlt') == 1);

sub menue() {
	
print $q->startform('POST',$q->url(),'application/x-www-form-urlencoded'),
	$q->h3("Einzahlung auf das Druckkostenkonto eines Benutzers"),
	  $q->p('Bitte geben Sie die Kennung des Benutzerkontos und die H&ouml;he der Einzahlung (in Seiten) an!'),
      $q->p('Benutzerkennung: ',$q->textfield(-name=>'kennung',-default=>'',-override=>1,-size=>8,-maxlength=>8)
      .'&nbsp;&nbsp;&nbsp;&nbsp;Einzahlung     : ',$q->textfield(-name=>'einzahlung',-default=>'0',-override=>1,-size=>4,-maxlength=>5)," Seiten",
      '<br>Verwendungszweck / Begr&uuml;ndung:',$q->textfield(-name=>'grund',-default=>'',-override=>1,-size=>60,-maxlength=>200)),
      $q->hidden('ausgewaehlt',1),
      $q->submit('submit',"Ausfhren"),'   ',$q->defaults('Zurcksetzen'),
      $q->p("<HR>"),
      $q->h3("Aktuelle Kontost&auml;nde"),
      tabelle(),
      $q->p("<small>Nur Kontost&auml;nde ungleich 0 Seiten werden angezeigt.</small>"),
      $q->endform();
print "</BODY></HTML>";
}

sub tabelle() {
        my ($name,$passwd,$uid,$gid,$quota,$comment,$gcos,$dir,$shell,$expire);
	my $ret = $q->p('<table border="1" width="80%"><tr><th>Benutzer</th><th align=right>Alter Kontostand</th><th align=right>Seitdem gedruckt</th><th align=right>Aktueller Kontostand</th></tr>');
	foreach (sort keys %USER) {
		($name,$passwd,$uid,$gid,$quota,$comment,$gcos,$dir,$shell,$expire)=getpwnam("$_");
		if (($USER{"$_"}->{'gedruckt'}>0) || ($USER{"$_"}->{'bezahlt'}!=0)) {
		$ret=$ret."<tr><td><a href=\"$_\@mariengymnasium-arnsberg.de\">$_</a> ($gcos)</td><td align=right>".$USER{"$_"}->{'bezahlt'}."</td><td align=right>".$USER{"$_"}->{'gedruckt'}."</td><td align=right>". (($USER{"$_"}->{'bezahlt'}-$USER{"$_"}->{'gedruckt'}<0)?"<font color=\"red\">":"<font color=\"black\">").($USER{"$_"}->{'bezahlt'}-$USER{"$_"}->{'gedruckt'})."</font></td></tr>\n";}
	}
	return $ret."</table>";
}

sub auswerten() {
	my $userid = lc($q->param('kennung'));
	my $einzahlung = $q->param('einzahlung');
	my $grund = $q->param('grund');
	my ($name,$passwd,$uid,$gid,$quota,$comment,$gcos,$dir,$shell,$expire) = getpwnam ("$userid");
	#my $name=$userid; my $comment="Kommentar";

	if ( (not defined $name) || ($name eq "") ) {
		print $q->p("Ein Benutzer mit der Kennung <font color=darkred>$userid</font> ist nicht bekannt!");
    } else {
    	if ($einzahlung != 0) {
    		if ( ($REMOTE =~ /^(deckersj)|(schmidtw)$/) || ($grund ne "") ) {
	    		$USER{$userid}->{'bezahlt'}+=$einzahlung;
	    		speichern();
		        open MAIL,"|$sendmail printmaster\@arktur" || warn "Konnte keine Mail an printmaster\@arktur versenden!\n";
		        #open MAIL,">> /bin/mailfile" || warn "Konnte keine Mail an $_\@arktur versenden!\n";
		        print MAIL "From: Druckkostenabrechnung <printmaster\@mariengymnasium-arnsberg.de>\n".
		        		   "To: Druckkostenabrechnung <printmaster\@mariengymnasium-arnsberg.de>\n".
		                   "Reply-To: Druckkostenabrechnung <printmaster\@mariengymnasium-arnsberg.de>\n".
		                   "Subject: Druckkostenverwaltung\n".
		                   "Content-type: text/plain; charset=ISO-8859-1\nContent-transfer-encoding: 8BIT\n".
		                   "X-mailer: pcost.cgi Web-Frontend zur Druckkostenerfassung (Joachim Deckers, <deckersj\@mariengymnasium-arnsberg.de)\n".
		                   "\nEinzahlung durch den Benutzer $REMOTE:\n\n",
		                   "  Benutzer: $userid ($gcos)\n  Einzahlung: $einzahlung\n  Grund: $grund\n\n";
		        print MAIL $USER{"$userid"}->ausgabe(0);
		        close MAIL;  
	    	    print $q->p("Die Einzahlung f&uuml;r Benutzer <font color=darkred>$name</font> ( $gcos ) wurde entgegengenommen.<hr>");
		&menue();
	    	 } else {
    			print $q->p("Bitte geben Sie eine Begr&uuml;ndung f&uuml;r die Freiseiten an!");
	    	 }	    	    
    	} else {
    		print $q->p("\"$einzahlung\" ist kein g&uuml;tiger Betrag f&uuml;r eine Einzahlung!");
    	}	
    }
	print "</BODY></HTML>";
}

sub speichern() {
	logrotate($dbfilename);
	open OUT,">>$dbfilename" || die "Fehler: Konnte Datenbank mit Einzahlungen ($dbfilename) nicht erstellen!\n";
	foreach (sort keys %USER) {
	  		print OUT "$_\t".$USER{"$_"}->{'bezahlt'}."\n";
	}
	close OUT;
}

sub logrotate() {
	my $filename=shift;
	my $i;
	my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks);
	if (defined $filename) {
		if (-f "$filename.9") {
			unlink "$filename.9" ||
			  die "Lschen nicht mglich: $filename.9\n";
		}	
		for ($i=8;$i>=0;--$i) {
			if (-f "$filename.$i") {
				rename("$filename.".$i,"$filename.".($i+1)) ||
				  print "<b>Fehler: rename fehlgeschlagen ($filename.$i,$filename.".($i+1)."</b></body></html>\n",  
				  die "rename nicht mglich: $filename.$i --> $filename.".($i+1)."\n";
			}
		}
		($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks)=stat("$filename");
		if (open OUT ,">>$filename") { print OUT "#\n# Logdatei geschlossen durch logrotate in pcost.pl (".localtime().")\n#\n"; close OUT;}
		rename ("$filename","$filename.0") || die "rename nicht mglich: $filename --> $filename.0\n";
		open OUT ,">$filename" || die "Erzeugen der Datei $filename war nicht mglich!\n";
		print OUT "#\n# erzeugt durch logrotate in pcost.pl (".localtime().")\n#\n";
		close OUT;
		chown $uid,$gid,$filename;
		chmod $mode & 07777,$filename;
	}
}
