/*
  lplog.c


  Liest den Seitenzaehler des Laserdruckers an /dev/lp0 aus.
  Voraussetzung: Drucker ist mit /dev/lp0 verbunden, ist eingeschaltet
  und unterstuetzt PJL (Printer Job Language).

  Der Seitenzaehler wird als String in eine Logdatei sowie als
  Returncode geliefert.

  Joachim Deckers <deckersj@mariengymnasium-arnsberg.de>
*/

#include "stdio.h"
#include "unistd.h"
#include "stdlib.h"
#include "time.h"

#define ST_READY      1
#define ST_PROCESSING 2
#define ST_OTHER      3

const char dev[]     = "/dev/lp0",
           logfile[] = "/var/log/pcost/pc-log",
           errfile[] = "/tmp/pc-err",
	   init[]    = "\033%%-12345X";
char       buf[200], *datetime, *username, *jobname, timebuf[60];
           
const long timeout = 30; /* Anzahl Sekunden fr einen erneuten Versuch, auf besetztes lp0 zuzugreifen*/

char *time2str(void) {
  time_t t=time(NULL);
  char *p = ctime(&t);
  p[strlen(p)-1]='\0';
  return p;
}

void wait(int sec) {
  time_t t=time(NULL);
  while (time(NULL)-t<sec) ;	
}

int check_status(FILE *fp, FILE *ferr) {
  time_t t;
  int ende=0, retval=0, l;
  
  *buf=0;
  fprintf (fp,init);
  fprintf (fp,"@PJL INFO STATUS\015\012");
  fprintf(fp,init);
  for (t=time(NULL);time(NULL)-t<=timeout && !ende;) { /* Bei Fehler keine Endlosschleife erzeugen! */
   l=strlen(buf);
   fgets((char *)(buf+l),200-l,fp);
   if (strlen(buf)>0 && l==strlen(buf)) { ende=1; } /* nichts neues gelesen */
   if (strstr(buf,"DISPLAY=")) {
      retval = strstr(buf,"Ready")?ST_READY: strstr(buf,"Processing") ? ST_PROCESSING : ST_OTHER;
   } 
  } 
  if (l>0) { fprintf(ferr,buf); fflush(ferr);}  
  return retval;
}

long pc(FILE *ferr, int start) {
  int i,naechste=0, busy=1;
  FILE *fp=NULL;
  time_t t=time(NULL);
  
  /* Zunaechst warten, BIS lp0 belegt ist */
  if(!start) {
  	fprintf(ferr,"  Warte auf Beginn des Druckauftrags (%s)...",time2str());
  	fflush(ferr);
  	do {
  		if (fp) { 
  		  busy=check_status(fp,ferr);
  		  fclose(fp);
  		  fp=NULL; 
  		  if (busy == ST_READY) { wait(5);} }
  		  fp=fopen("/dev/lp0","r+");  		
  	} while (fp && (time(NULL)-t<timeout));  	
  	fprintf(ferr,"okay (%s).\n",time2str());
  	if (fp && !busy) { fclose(fp); fp=NULL; fprintf(ferr,"  Fehler: Device nicht besetzt! (%s)\n",time2str()); }
  	fflush(ferr);
  }
  	
  t=time(NULL); /* Zeit neu messen */
  while ((!fp) && (time(NULL)-t<=timeout*2)) {
    fp=fopen(dev,"r+"); 
  }  
  if (!fp) {
    fprintf(ferr,"Fehler: Device %s konnte nicht geoeffnet werden (timeout: %ld)!\n",dev,time(NULL)-t);
    exit(0);
  }
  
//  t=time(NULL); /* Zeit neu messen */
//  while (check_status(fp,ferr) == ST_PROCESSING && time(NULL)-t<=timeout*5) ;
  	
  fprintf (fp,init);
  fprintf (fp,"@PJL INFO PAGECOUNT\015\012");
  fprintf(fp,init);
  for (t=time(NULL);time(NULL)-t<=timeout;) { /* Bei Fehler keine Endlosschleife erzeugen! */
   fgets((char *)buf,200,fp);
   if (naechste) {
     fclose(fp);
     return (strtol(buf,NULL,10));
   }
   if (strstr(buf,"@PJL INFO PAGECOUNT")) {
      naechste = 1;
   } else { naechste = 0; } 
  } 
  
  fclose(fp);
  return -1;
}

void evalpar(int argc, char **argv) {
  int i; char opt;
  
  for (i=1; i<argc; ++i) {
  	if (strlen(argv[i])>=2) {
  	  opt = (argv[i])[1];
  	  switch(opt) {
  	    case 'n': username = &argv[i][2]; break;
  	    case 't': datetime = &argv[i][2]; break;
	    case 'J': jobname  = &argv[i][2]; break;  	    
  	  }
  	}
  }
}


int main(int argc, char **argv) {
  FILE *fp, *ferr;
  int i,naechste=0;
   time_t t=time(NULL);
  long p1, p2;

  evalpar(argc,argv);
  ferr=fopen(errfile,"a");; 
  if (!ferr) {
    fprintf(stderr,"Fehler: Fehlerdatei %s konnte nicht geoeffnet werden!\n",errfile);
    exit(0);
  }
  
  fprintf(ferr,"Aufruf %s um %s",argv[0],ctime(&t));
  fflush(ferr);
  fp=fopen(logfile,"a");; 
  if (!fp) {
    fprintf(ferr,"Fehler: Logdatei %s konnte nicht geoeffnet werden!\n",logfile);
    exit(0);
  }
  p1 = pc(ferr,strstr(argv[0],"start"));
  fprintf(fp,"%s:\t%s\t%s\t%ld\t%s\n",strstr(argv[0],"start")?"jobstart":"jobend",(username)?username:"unbekannt",(datetime)?datetime:"unbekannt",p1,(jobname)?jobname:"unbekannt");  	
   
  fclose(fp);
  fclose(ferr);
  return 0;  
}
