#!/usr/bin/perl
#
#
# Druckkostenabrechnung fuer print-n21
# Joachim Deckers <deckersj@mariengymnasium-arnsberg.de>
#

use strict;

# ----------------------------------
package job;

local ($job::counter);

sub new {
	my $class=shift;
	my $self = {};
	$self->{'tag'}=shift;
	$self->{'zeit'}=shift;
	$self->{'vorher'}=shift;
	$self->{'nachher'}=shift;
	$self->{'naechster'}=shift;
	if ($self->{'vorher'}>=0) {
		if ($self->{'naechster'}>=0) {
			$self->{'gedruckt'}=$self->{'naechster'}-$self->{'vorher'};
		} else {
			if ($self->{'nachher'}>=0) {
				$self->{'gedruckt'}=$self->{'nachher'}-$self->{'vorher'};
				if ($self->{'gedruckt'}==0) {$self->{'gedruckt'}='unbekannte Anzahl';}
			} else {
				$self->{'gedruckt'}='unbekannte Anzahl';
			}
		}			
	} else {
		$self->{'gedruckt'}='unbekannte Anzahl';	
	}
		
	$job::counter++;
	bless($self,$class);
}

sub ausgabe {
	my $self=shift;
	my $ausfuehrlich = shift;
	$ausfuehrlich = 0 if not defined $ausfuehrlich;
	
	if (! $ausfuehrlich) { return "  $self->{'tag'}, $self->{'zeit'}: $self->{'gedruckt'} Seiten\n"; }
	else { return "  $self->{'tag'}, $self->{'zeit'}: $self->{'gedruckt'} Seiten ($self->{'vorher'},$self->{'nachher'}|$self->{'naechster'})\n"; }
}

sub gedruckt {
	my $self=shift;
	return ($self->{'gedruckt'} !~ 'unbek') ? $self->{'gedruckt'} : 0;
}

sub DESTROY { $job::counter-- };

# ----------------------------------
package userentry;

local ($userentry::counter);

sub new {
	my $class=shift;
	my $self = {};
	$self->{'name'}=shift;
	$self->{'bezahlt'}=shift;
	$self->{'gedruckt'}=0;
	$self->{'anzjobs'}=0;
	$self->{'jobs'}=();
	$userentry::counter++;
	bless($self,$class);
}

sub einzahlung {
	my $self = shift;
	my $anz = shift;
	$self->{'bezahlt'}+=$anz;
}

sub neuerjob {
	my $self = shift;
	my $job = shift;
	my $num = $self->{'anzjobs'};
	$self->{'jobs'}[$num]=$job;
	$self->{'gedruckt'} += ($self->{'jobs'}[$num])->gedruckt();
	$self->{'anzjobs'}++;
}

sub ausgabe {
	my $self=shift;
	my $AUS=shift;
	my $ausfuehrlich=shift;
	my ($i,$job,$ret);
	
	$ausfuehrlich=0 if not defined $ausfuehrlich;
	$ret = "User: ".$self->{'name'}."\n".
	"  Alter Kontostand : ".$self->{'bezahlt'}." Seiten.\n".
	"  Danach gedruckt  : ".$self->{'gedruckt'}." Seiten in ".$self->{'anzjobs'}." Druckauftrgen.\n".
	"  Neuer Kontostand : ".($self->{'bezahlt'}-$self->{'gedruckt'})." Seiten.\n".
	"  Liste der Druckauftrge:\n";
	for ($i=0;$i<$self->{'anzjobs'};++$i) {
		$job = $self->{'jobs'}[$i];
		$ret = $ret.($job->ausgabe($ausfuehrlich));
	}
	return $ret;
}

sub DESTROY { $userentry::counter-- };

# ----------------------------------
package main;

# Aufbau des Hashs:
# username, [username, bezahlt, seitengedruckt, anzeintrge, eintrag1, eintrag2, ...]
#  Eintrag:   [tag, zeit, seitenstart, seitenende, naechsterstart]

my %USER;
my $logfilename = "/var/log/pcost/pc-log";
my $dbfilename = "/var/local/pcost-db";
my ($user,$einzahlung,$startend,$zeitstempel,$pc,$ostartend,$jobname,$ojobname,$ouser,$opc,$nachher,$firstpc,$lastpc,$j,$m,$t,$tag,$zeit,$stand,$summe);
my $sendmail = "/usr/sbin/sendmail";
# my $sendmail = "/bin/cat";

# Hash mit Daten aus der Datenbank fllen
if (open DB,"<$dbfilename") {
	while (<DB>) {
		chomp;
		if (! /^#/) {
			($user,$einzahlung) = split /\t/;
			if (not defined $USER{"$user"}) {
				$USER{"$user"}=userentry->new($user,$einzahlung);
			} else {
				$USER{"$user"}->{'bezahlt'} += $einzahlung;
			}
		}
	}
	close DB;
} else {
	print STDERR "Konnte Datenbank nicht ffnen: ".$dbfilename."\n";
}

$ouser=""; $ostartend=""; $opc=0; $nachher=0;

# Hash mit Daten aus dem Logbuch fllen
if (open LOGF,"<$logfilename") {
	while (<LOGF>) {
		chomp;
		if (! /^#/) { # Kommentarzeilen berlesen!
			($startend,$user,$zeitstempel,$pc,$jobname) = split /\t/;
			if (not defined $firstpc) { $firstpc=$pc; }
			$lastpc=$pc;
			$stand=$zeitstempel;
			($j,$m,$t,$zeit)=split /\-/,$zeitstempel,4;
			$tag="$t.$m.$j";
			$zeit=~s/\.\d\d\d$//;
			if (not defined $USER{"$user"}) { $USER{"$user"}=userentry->new($user,0); }
			if ($ostartend =~ /start/) { # Job ist gestartet worden 
			  if ($startend =~ /start/) { # erneuter Jobstart (vor Jobende) 
			    if ("$ouser" ne "") {
                              $USER{"$ouser"}->neuerjob(job->new("$tag","$zeit",$opc,$nachher,$pc));
                            }
			    $ouser=$user;$opc=$pc;$ojobname=$jobname;
			  } else { # Jobende (normal, nach Jobstart)
			      $nachher=$pc;	
			      $ouser=$user;
                              $USER{"$ouser"}->neuerjob(job->new("$tag","$zeit",$opc,$nachher,$pc));
                              $ouser="";
			  }
			} else { # also hoffentlich erster job 
				$ouser=$user; $ostartend=$startend;$opc=$pc;$ojobname=$jobname;
			}
		}
	}
	# Den letzten Job auch erfassen!!!	
	if ($ostartend =~ /start/) { # Job ist gestartet worden 
	  if ("$ouser" ne "") {
            $USER{"$ouser"}->neuerjob(job->new("$tag","$zeit",$opc,$nachher,$pc));
	  } 
        }
	close LOGF;
} else {
	print STDERR "Konnte Logbuch nicht ffnen: ".$logfilename."\n";
}


$summe=0;
foreach (sort keys %USER) {
	if ($USER{"$_"}->{'gedruckt'}>0) {
		$summe+=$USER{"$_"}->{'gedruckt'};
        print $USER{"$_"}->ausgabe(1); 
        open MAIL,"|$sendmail $_\@arktur,printmaster\@arktur" || warn "Konnte keine Mail an $_\@arktur versenden!\n";
        #open MAIL,">> /bin/mailfile" || warn "Konnte keine Mail an $_\@arktur versenden!\n";
        print MAIL "From: Druckkostenabrechnung <printmaster\@mariengymnasium-arnsberg.de>\n".
        		   "To: $_\@mariengymnasium-arnsberg.de\n".
                   "Reply-To: Druckkostenabrechnung <printmaster\@mariengymnasium-arnsberg.de>\n".
                   "Subject: Woechentliche Druckkostenabrechnung\n".
                   "Content-type: text/plain; charset=ISO-8859-1\nContent-transfer-encoding: 8BIT\n".
                   "X-mailer: pcost.pl Skript zur Druckkostenerfassung (Joachim Deckers, <deckersj\@mariengymnasium-arnsberg.de)\n".
                   "\nWchentliche Druckkostenabrechnung fr den lokalen Benutzer $_:\n\n";
        print MAIL $USER{"$_"}->ausgabe(0);
        print MAIL "\nBitte sorge/sorgen Sie umgehend fr einen Ausgleich deines/Ihres Druckkostenkontos.\nRckfragen und Einwendungen gegen diese Abrechnung bitte an printmaster\@mariengymnasium-arnsberg.de.\n" 
          if (($USER{"$_"}->{'bezahlt'} - $USER{"$_"}->{'gedruckt'})<0);
        close MAIL;  
    }    
}
print "\n\nPagecounter Anfang            : $firstpc\n".
	  "Pagecounter Ende              : $lastpc\n".
	  "Tatseachlich gedruckte Seiten : ".($lastpc-$firstpc)."\n".
	  "Summe aller erfassten Seiten  : $summe\n".
	  "Nicht erfasste Seiten         : ".($lastpc-$firstpc-$summe)."\n";
logrotate($logfilename);
logrotate($dbfilename);
open OUT,">>$dbfilename" || die "Fehler: Konnte Datenbank mit Einzahlungen ($dbfilename) nicht erstellen!\n";
foreach (sort keys %USER) {
	if (($USER{"$_"}->{'bezahlt'} - $USER{"$_"}->{'gedruckt'}) != 0) {
	  print OUT "$_\t".($USER{"$_"}->{'bezahlt'} - $USER{"$_"}->{'gedruckt'})."\n";}
}
close OUT;


#
# ENDE
#

sub logrotate() {
	my $filename=shift;
	my $i;
	my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks);

	if (defined $filename) {
		if (-f "$filename.9") {
			unlink "$filename.9" ||
			  die "Lschen nicht mglich: $filename.9\n";
		}	
		for ($i=8;$i>=0;--$i) {
			if (-f "$filename.$i") {
				rename ("$filename.".$i,"$filename.".($i+1)) || 
				  die "rename nicht mglich: $filename.$i --> $filename.".($i+1)."\n";
			}
		}
		($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks)=stat("$filename");
		if (open OUT ,">>$filename") { print OUT "#\n# Logdatei geschlossen durch logrotate in pcost.pl (".localtime().")\n#\n"; close OUT;}
		rename ("$filename","$filename.0") || die "rename nicht mglich: $filename --> $filename.0\n";
		open OUT ,">$filename" || die "Erzeugen der Datei $filename war nicht mglich!\n";
		print OUT "#\n# erzeugt durch logrotate in pcost.pl (".localtime().")\n#\n";
		close OUT;
		chown $uid,$gid,$filename;
		chmod $mode & 07777,$filename;
	}
}
